///////////////////////////////////////////////////////////////////////////////
// Enum.h by Jeff Fitzsimons (2002)
//
// This file contains the declaration of a simple data-oriented class used
// to cache a list of all the windows on the system.  Note that it's overkill
// for this particular application in that it stores extra data and, by
// defining RECUSIVE_ENUMERATION, it will build a tree-like list of _all_
// windows on the system.
//
// For simplicity, all the code appears within this header file.  There are
// no external definitions.
///////////////////////////////////////////////////////////////////////////////

#ifndef ENUM_H_89880206_9D49_4495_9EE9_08C9834907C1__
#define ENUM_H_89880206_9D49_4495_9EE9_08C9834907C1__

#pragma once

#include <vector>

// EnumChildWndCallback is called by the system after having called EnumWindows:
BOOL __stdcall EnumChildWndCallback(HWND hWnd, LPARAM lParam);

// Forward reference for self-referential structure.
struct WNDINFO;
typedef std::vector<WNDINFO> WNDLIST, *PWNDLIST;

typedef std::basic_string<TCHAR> TSTRING;

// WNDINFO is used to cache the data for a single window:
struct WNDINFO
{
	HWND hWnd;					// The window handle.
	HINSTANCE hInstance;
	DWORD dwStyle;				// Window style, see Get/SetWindowLong.
	DWORD dwStyleEx;			// Window extended style, see Get/SetWindowLong.

	TSTRING strTitle;			// The window's title, used in the list windows.
	TSTRING strClassname;	// The window's class name (see RegisterClass)
#ifdef RECURSIVE_ENUMERATION
	PWNDLIST plstChildren;	// A list of this window's child windows.
#endif

	BOOL bDirty;				// TRUE if the properties have been changed w/o being committed.  Used fairly incompletely.  :-)

	WNDINFO(const WNDINFO &other) :
		hWnd(NULL), hInstance(NULL), dwStyle(0), 
		dwStyleEx(0), strTitle(), strClassname(), 
#ifdef RECURSIVE_ENUMERATION
		plstChildren(NULL),
#endif
		bDirty(FALSE)
	{
		operator =(other);
	}

	WNDINFO() : hWnd(NULL), hInstance(NULL), dwStyle(0), 
		dwStyleEx(0), strTitle(), strClassname(), 
#ifdef RECURSIVE_ENUMERATION
		plstChildren(NULL), 
#endif
		bDirty(FALSE) { }

	~WNDINFO() 
	{ 
#ifdef RECURSIVE_ENUMERATION
		if (plstChildren)
		{
			plstChildren->clear();
			
			delete plstChildren; 
			plstChildren = NULL;
		}
#endif
	}

	operator =(const WNDINFO &other)
	{
		hWnd = other.hWnd;
		hInstance = other.hInstance;
		dwStyle = other.dwStyle;
		dwStyleEx = other.dwStyleEx;

		strTitle = other.strTitle;
		strClassname = other.strClassname;
		bDirty = other.bDirty;

#ifdef RECURSIVE_ENUMERATION
		delete plstChildren;

		if (other.plstChildren)
			plstChildren = new WNDLIST(*other.plstChildren);
		else
			plstChildren = NULL;
#endif
	}
};

BOOL __stdcall EnumChildWndCallback(HWND hWnd, LPARAM lParam)
{
	PWNDLIST plstWindows = (PWNDLIST)lParam;
	WNDINFO wndInfo;

	wndInfo.hWnd = hWnd;

	TCHAR arBuffer[1024];

	ZeroMemory(arBuffer, sizeof(arBuffer));
	if (GetWindowText(hWnd, arBuffer, sizeof(arBuffer) - 1) > 0)
		wndInfo.strTitle = arBuffer;

	ZeroMemory(arBuffer, sizeof(arBuffer));
	if (GetClassName(hWnd, arBuffer, sizeof(arBuffer) - 1) > 0)
		wndInfo.strClassname = arBuffer;

	wndInfo.hInstance = (HINSTANCE)::GetWindowLong(hWnd, GWL_HINSTANCE);
	wndInfo.dwStyle = (DWORD)::GetWindowLong(hWnd, GWL_STYLE);
	wndInfo.dwStyleEx = (DWORD)::GetWindowLong(hWnd, GWL_EXSTYLE);

#ifdef RECURSIVE_ENUMERATION
	wndInfo.plstChildren = new WNDLIST;

	EnumChildWindows(hWnd, EnumChildWndCallback, (LONG)wndInfo.plstChildren);

	if (wndInfo.plstChildren->empty())
	{
		delete wndInfo.plstChildren;
		wndInfo.plstChildren = NULL;
	}
#endif

	plstWindows->push_back(wndInfo);

	return TRUE;
}

#endif
