///////////////////////////////////////////////////////////////////////////////
// FireThread.h by Jeff Fitzsimons (2002)
//
// This file contains the core functionality for allocating memory and creating
// a thread within the address space of another running process.
//
// For simplicity, all the code appears within this header file.  There are
// no external definitions.
///////////////////////////////////////////////////////////////////////////////

#ifndef FIRETHREAD_H_69AC3101_C5B1_47E5_BF8C_9846CDA27985__
#define FIRETHREAD_H_69AC3101_C5B1_47E5_BF8C_9846CDA27985__

#pragma once

// A utility function written by Jeff Richter.  Enables DEBUG privileges on the
// process indicated by hToken.
BOOL EnablePrivilege(HANDLE hToken, LPCTSTR szPrivName, BOOL fEnable);

// ShowErrorBox shows an error dialog by calling FormatMessage on the specified
// Win32 error code:
VOID ShowErrorBox(PCSTR szMessage, DWORD dwError);

// ShowLastError box simply calls the above function with GetLastError() as the
// Win32-error argument:
VOID ShowLastErrorBox(PCSTR szMessage);

// FireRemoteThread does the dirty work of allocating memory within the foreign
// process and copying our thread function into its address space.  Since this
// example is geared entirely toward setting the LAYERED style and calling
// SetLayeredWindowAttributes, all the arguments reflect this goal:
BOOL FireRemoteThread(HANDLE hProcess, HWND hWnd, BOOL bIsLayered, DWORD dwLayeredFlags, BYTE bOpacity, COLORREF crTransparent)
{
	HANDLE hRemoteThread = NULL;	// after creating the remote thread, we'll wait for it to terminate so we can clean up.
	PVOID pRemoteMem = NULL;		// will point to an allocated memory block within the remote process.
	HANDLE hToken = NULL;			// used to get the process' access token.

	BOOL bResult = FALSE;

	try
	{
		if (OpenProcessToken(GetCurrentProcess(), TOKEN_ADJUST_PRIVILEGES, &hToken)) 
		{
			if (!EnablePrivilege(hToken, SE_DEBUG_NAME, TRUE)) 
				throw;
		}
		else
			throw;

		// Allocate space for the data and for the function itself all at once...
		DWORD dwTotalSize = sizeof(REMOTE_THREAD_DATA) + MAXTHREADLENGTH;

		pRemoteMem = VirtualAllocEx( hProcess, 0, dwTotalSize, MEM_COMMIT, PAGE_EXECUTE_READWRITE );
		if ( pRemoteMem == 0 )
		{
			ShowLastErrorBox(_T("VirtualAlloc of RemoteMemory failed."));
			throw;
		}

		// The function will be copied in immediately after the thread data...
		PVOID pRemoteFn = (PVOID)((PCHAR)pRemoteMem + sizeof(REMOTE_THREAD_DATA));

		REMOTE_THREAD_DATA ThreadData;

		ThreadData.hWndTarget = hWnd;
		ThreadData.fLayered = bIsLayered;
		ThreadData.dwLayeredFlags = dwLayeredFlags;
		ThreadData.bOpacity = bOpacity;
		ThreadData.crTransparent = crTransparent;
		ThreadData.dwError = 0;

		// Copy in the full path to User32.dll so that the remote thread has a valid
		// string to pass into LoadLibrary()...
		GetModuleFileName(GetModuleHandle(_T("USER32.DLL")), ThreadData.szUser32Filename, sizeof(ThreadData.szUser32Filename));

		HMODULE hKernel32 = GetModuleHandle(_T("KERNEL32.DLL"));

		// Set the proc addresses for all the critical functions within Kernel32.dll...
		ThreadData.DynLoadLibrary = (LOADLIBRARY)GetProcAddress(hKernel32, "LoadLibraryA");
		_ASSERT(ThreadData.DynLoadLibrary != NULL);

		ThreadData.DynGetProcAddress = (GETPROCADDRESS)GetProcAddress(hKernel32, "GetProcAddress");
		_ASSERT(ThreadData.DynGetProcAddress != NULL);

		ThreadData.DynFreeLibrary = (FREELIBRARY)GetProcAddress(hKernel32, "FreeLibrary");
		_ASSERT(ThreadData.DynFreeLibrary != NULL);

		ThreadData.DynGetLastError = (GETLASTERROR)GetProcAddress(hKernel32, "GetLastError");
		_ASSERT(ThreadData.DynGetLastError != NULL);

		if (!WriteProcessMemory( hProcess, pRemoteMem, &ThreadData, sizeof(REMOTE_THREAD_DATA), 0))
		{
			ShowLastErrorBox(_T("WriteProcessMemory on remote Data failed"));
			throw;
		}

		// Copy the thread function in immediately after the data structure...
		if (!WriteProcessMemory( hProcess, pRemoteFn, RemoteThreadProc, MAXTHREADLENGTH, 0))
		{
			ShowLastErrorBox(_T("WriteProcessMemory of RemoteMemory failed."));
			throw;
		}

		// Fire up the freshly-copied remote thread!
		DWORD dwThreadID;
		hRemoteThread = CreateRemoteThread( 
			hProcess,												// Handle to the foreign process.
			NULL,														// Security attributes.  NULL for default.
			0,															// Initial stack size.  0 for default.
			(DWORD (__stdcall *)( void *)) pRemoteFn,		// The thread address within the other process.
			pRemoteMem,												// The argument (LPVOID) to pass into the thread proc.
			0,															// Creation flags.  CREATE_SUSPENDED or 0.
			&dwThreadID );											// The thread ID.

		if ( hRemoteThread == NULL )
		{
			ShowLastErrorBox(_T("CreateRemoteThread failed."));
			throw;
		}

		// Wait for up to 30 seconds for the remote thread to terminate...
		DWORD dwResult = WaitForSingleObject( hRemoteThread, 30 * 1000 );

		switch ( dwResult )
		{
			case WAIT_TIMEOUT:
				MessageBox(NULL, _T("The thread did not terminate within the allotted time."), _T("Thread Error"), MB_OK);
				throw;

			case WAIT_FAILED:
				ShowLastErrorBox(_T("Wait failed"));
				throw;

			case WAIT_OBJECT_0:
				{	// avoids C2361

					DWORD dwExitCode = -1;
					GetExitCodeThread(hRemoteThread, &dwExitCode);

					// If the thread didn't return a success code (0 in this case), show
					// some diagnostic information...
					if (dwExitCode != 0)
					{
						// Retrieve the REMOTE_THREAD_DATA structure...
						if (!ReadProcessMemory( hProcess, pRemoteMem, &ThreadData, sizeof(ThreadData), 0))
						{
							ShowLastErrorBox(_T("Unable to retrieve thread termination error"));
							throw;
						}

						// Format a message based on the error code returned by the thread...
						LPVOID lpMsgBuf;
						FormatMessage( 
							FORMAT_MESSAGE_ALLOCATE_BUFFER | 
							FORMAT_MESSAGE_FROM_SYSTEM | 
							FORMAT_MESSAGE_IGNORE_INSERTS,
							NULL,
							ThreadData.dwError,
							MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
							(LPTSTR) &lpMsgBuf,
							0,
							NULL 
							);

						TCHAR arBuffer[255];
						_stprintf(arBuffer, _T("Thread terminated with exit code %d\n%Message ID %d:  %s"), dwExitCode, ThreadData.dwError, (LPTSTR)lpMsgBuf);

						LocalFree( lpMsgBuf );

						MessageBox(NULL, arBuffer, _T("Thread Error"), MB_OK);
					}
					else
						bResult = TRUE;		// Everything executed smoothly.
				}

				break;

			default:
				MessageBox(NULL, _T("Wait terminated with unknown code."), _T("Error"), MB_OK);
				break;
		}
	}
	catch (...)
	{
		MessageBox(NULL, _T("Unknown error thrown."), _T("Error"), MB_OK);
	}

	////////////////////////////////////////////////////////
	// Clean up, free the allocated resources...

	if (hRemoteThread)
		CloseHandle(hRemoteThread);

	if (pRemoteMem)
		VirtualFreeEx( hProcess, pRemoteMem, 0, MEM_RELEASE );

	if (hToken)
		CloseHandle(hToken);

	return bResult;
}

///////////////////////////////////////////////////////////////////////////////
// From:  EnableDebugPrivAndRun.cpp
// Notices: Written 1998 by Jeffrey Richter
// Description: Enables the Debug privilege before running an app
///////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////
 
BOOL EnablePrivilege(HANDLE hToken, LPCTSTR szPrivName, BOOL fEnable) 
{
	TOKEN_PRIVILEGES tp;
	tp.PrivilegeCount = 1;
	LookupPrivilegeValue(NULL, szPrivName, &tp.Privileges[0].Luid);
	tp.Privileges[0].Attributes = fEnable ? SE_PRIVILEGE_ENABLED : 0;
	AdjustTokenPrivileges(hToken, FALSE, &tp, sizeof(tp), NULL, NULL);
	return((GetLastError() == ERROR_SUCCESS));
}

VOID ShowErrorBox(PCSTR szMessage, DWORD dwError)
{
	// FormatMessage will get allocate & fill a string with an error message
	// for the supplied Win32 error code...
	LPVOID lpMsgBuf;
	FormatMessage( 
		FORMAT_MESSAGE_ALLOCATE_BUFFER | 
		FORMAT_MESSAGE_FROM_SYSTEM | 
		FORMAT_MESSAGE_IGNORE_INSERTS,
		NULL,
		dwError,
		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
		(LPTSTR) &lpMsgBuf,
		0,
		NULL 
		);

	TCHAR arBuffer[1024];
	_stprintf(arBuffer, _T("%s:\n%s"), szMessage, (LPTSTR)lpMsgBuf);

	LocalFree( lpMsgBuf );

	::MessageBox(NULL, arBuffer, _T("Error"), MB_OK | MB_ICONERROR);
}

VOID ShowLastErrorBox(PCSTR szMessage)
{
	ShowErrorBox(szMessage, GetLastError());
}

#endif
