///////////////////////////////////////////////////////////////////////////////
// RemoteThread.h by Jeff Fitzsimons (2002)
//
// This file contains the actual thread which will be launched within the
// address space of another running process.  It simply sets the WS_EX_LAYERED
// extended style and calls SetLayeredWindowAttributes.
//
// For simplicity, all the code appears within this header file.  There are
// no external definitions.
///////////////////////////////////////////////////////////////////////////////

#ifndef REMOTETHREAD_H_12ECEDC4_8614_47C9_BA4F_77C7C5809BC7__
#define REMOTETHREAD_H_12ECEDC4_8614_47C9_BA4F_77C7C5809BC7__

#pragma once

///////////////////////////////////////////////////////////////////////////////
// I don't want to link in code that will keep the program from running
// at all on older systems, so I'm going to manually define these values.
#ifndef WS_EX_LAYERED
	#define WS_EX_LAYERED	( 0x00080000 )
#endif

// LWA_COLORKEY indicates a specified color should become transparent.
#ifndef LWA_COLORKEY
	#define LWA_COLORKEY		( 0x00000001 )
#endif

// LWA_ALPHA indicates the window should be 'translucent' proportional to a value 0..255.
#ifndef LWA_ALPHA
	#define LWA_ALPHA			( 0x00000002 )
#endif

///////////////////////////////////////////////////////////////////////////////
// Some basic, required function prototypes.  These functions all reside within
// Kernel32.DLL and will therefore be present on almost all running processes.
// This means that these functions will reside at the same address in virtually
// all processes, so our remote thread can rely on these to work.
typedef WINBASEAPI HMODULE (WINAPI *LOADLIBRARY)(LPCTSTR);					// ::LoadLibrary
typedef WINBASEAPI FARPROC (WINAPI *GETPROCADDRESS)(HMODULE, LPCTSTR);	//	::GetProcAddress
typedef WINBASEAPI BOOL (WINAPI *FREELIBRARY)(HMODULE);						//	::FreeLibrary
typedef WINBASEAPI DWORD (WINAPI *GETLASTERROR)(VOID);						//	::GetLastError
typedef WINBASEAPI HMODULE (WINAPI *GETMODULEHANDLE)(LPCTSTR);				// ::GetModuleHandle

///////////////////////////////////////////////////////////////////////////////
// Some function prototypes for User32.DLL.
typedef WINUSERAPI LONG (WINAPI *SETWINDOWLONG)(HWND, INT, LONG);	// ::SetWindowLong
typedef WINUSERAPI LONG (WINAPI *GETWINDOWLONG)(HWND, INT);			// ::GetWindowLong
typedef WINUSERAPI BOOL (WINAPI *SETWINDOWPOS)(HWND, HWND, INT, INT, INT, INT, UINT); // ::SetWindowPos
typedef WINUSERAPI BOOL (WINAPI *SETLAYEREDWINDOWATTRIBUTES)(HWND, COLORREF, BYTE, DWORD); // ::SetLayeredWindowAttributes

// This is a guess as to how long the thread might be in memory.  This is necessary
// so we can allocate the required memory for it in the foreign process.  Debug
// builds are significantly larger than Release builds, and these numbers must
// reflect that.
#ifdef DEBUG
	const DWORD MAXTHREADLENGTH = 1024 * 40;
#else
	const DWORD MAXTHREADLENGTH = 1024 * 16;
#endif

// REMOTE_THREAD_DATA holds all the information required to tell
// RemoteThreadProc what to do.
struct REMOTE_THREAD_DATA
{
	HWND		hWndTarget;			// The target window.
	BOOL		fLayered;			// Should the window be layered?

	DWORD		dwLayeredFlags;	// One or both of { LWA_COLORKEY, LWA_ALPHA }

	BYTE		bOpacity;			// If LWA_ALPHA set, this indicates the alpha level 0..255.
	COLORREF	crTransparent;		// If LWA_COLORKEY set, this indicates the 'transparent' color.

	TCHAR		szUser32Filename[MAX_PATH];	// The filename for User32.dll, including path.
														// this is required because we can't use string
														// literals within our thread function!

	DWORD		dwError;				// If an error occurrs, this will contain the error code returned by GetLastError.

	// Function pointers for the most basic required functions.  These are
	// all in Kernel32.DLL and will therefore appear at the same address
	// from _any_ app that loads Kernel32.DLL.  Very few apps do not load
	// it and those that have windows will have to load it.
	LOADLIBRARY			DynLoadLibrary;
	GETPROCADDRESS		DynGetProcAddress;
	FREELIBRARY			DynFreeLibrary;
	GETLASTERROR		DynGetLastError;

};

// Function ordinals that are different for UNICODE builds:
#ifdef UNICODE
	#define GETWINDOWLONG_ORDINAL 348
	#define SETWINDOWLONG_ORDINAL 607
#else
	#define GETWINDOWLONG_ORDINAL 347
	#define SETWINDOWLONG_ORDINAL 606
#endif

// User32.dll:  SetLayeredWindowAttributes
#define SETLAYEREDWINDOWATTRIBUTES_ORDINAL 569

// Error codes returned by the remote thread:
#define ERROR_UNKNOWN						1000
#define ERROR_USER32_LOAD					2000
#define ERROR_USER32_FUNCTION				3000
#define ERROR_USER32_FUNCTION_CALL		4000
#define ERROR_KERNEL32_FUNCTION_CALL	5000

static DWORD WINAPI RemoteThreadProc(LPVOID lpParameter)
{
	// Cast a pointer to the thread data passed in by the CreateRemoteThread() call...
	REMOTE_THREAD_DATA *pData = (REMOTE_THREAD_DATA *)lpParameter;

	// Wrapping everything in a try block will help prevent crashing the
	// other application.
	try
	{
		// Attempt to load User32.DLL:
		HMODULE hUser32 = pData->DynLoadLibrary(pData->szUser32Filename);
		// Note that, as Joe Chakra pointed out, we cannot use use this form:
		//		HMODULE hUser32 = pData->DynLoadLibrary("User32.DLL");
		// since the string literal "User32.DLL" would only exist within
		// the application into which this function is compiled.  Duh!

		if (hUser32 == NULL)
		{
			pData->dwError = pData->DynGetLastError();
			return ERROR_USER32_LOAD;
		}

		GETWINDOWLONG		DynGetWindowLong;
		SETWINDOWLONG		DynSetWindowLong;

		DynGetWindowLong = (GETWINDOWLONG)pData->DynGetProcAddress(hUser32, MAKEINTRESOURCE(GETWINDOWLONG_ORDINAL));
		DynSetWindowLong = (SETWINDOWLONG)pData->DynGetProcAddress(hUser32, MAKEINTRESOURCE(SETWINDOWLONG_ORDINAL));

		if ( (DynSetWindowLong == NULL) || (DynGetWindowLong == NULL) )
		{
			pData->dwError = pData->DynGetLastError();
			pData->DynFreeLibrary(hUser32);					// clean up
			return ERROR_USER32_FUNCTION;
		}

		// Get the current extended window style for comparison purposes...

		DWORD dwExOldStyle = DynGetWindowLong(pData->hWndTarget, GWL_EXSTYLE);

		if (dwExOldStyle == 0)
		{
			pData->dwError = pData->DynGetLastError();
			pData->DynFreeLibrary(hUser32);					// clean up
			return ERROR_USER32_FUNCTION_CALL;
		}

		// If we're going to set the layered bit and it's already set,
		// we have to make sure it gets turned off first in order for
		// any changes to take effect!  (see SetLayeredWindowAttributes).
		if (pData->fLayered && (dwExOldStyle & WS_EX_LAYERED) == WS_EX_LAYERED)
		{
			// From the MSDN:
			// "Note that once SetLayeredWindowAttributes has been called for a
			//  layered window, subsequent UpdateLayeredWindow calls will fail 
			//  until the layering style bit is cleared and set again."
			DynSetWindowLong(pData->hWndTarget, GWL_EXSTYLE, dwExOldStyle & ~WS_EX_LAYERED);
		}

		// Based upon our arguments and the current window style, figure
		// out what the new extended window style should be...

		DWORD dwExNewStyle = dwExOldStyle;

		if (pData->fLayered)
		{
			dwExNewStyle |= WS_EX_LAYERED;
		}
		else
			dwExNewStyle &= ~WS_EX_LAYERED;

		DynSetWindowLong(pData->hWndTarget, GWL_EXSTYLE, dwExNewStyle);

		// If we have set the LAYERED style, go ahead and set the layered
		// attributes...
		if (pData->fLayered)
		{
			SETLAYEREDWINDOWATTRIBUTES DynSetLayeredWindowAttributes = 
				(SETLAYEREDWINDOWATTRIBUTES)
				pData->DynGetProcAddress(
					hUser32, 
					MAKEINTRESOURCE(SETLAYEREDWINDOWATTRIBUTES_ORDINAL));

			if (DynSetLayeredWindowAttributes == NULL)
			{
				pData->dwError = pData->DynGetLastError();
				pData->DynFreeLibrary(hUser32);					// clean up
				return ERROR_KERNEL32_FUNCTION_CALL;
			}
			else
			{
				// The goal!  Call SetLayeredWindowAttributes on the window.
				DynSetLayeredWindowAttributes(
					pData->hWndTarget,				// target window
					pData->crTransparent,			// transparent color (used if dwLayeredFlags & LWA_COLORREF)
					pData->bOpacity,					// alpha value (used if dwLayeredFlags & LWA_ALPHA)
					pData->dwLayeredFlags);			// the layered flag(s) (any of { LWA_COLORREF, LWA_ALPHA })
			}
		}

		pData->DynFreeLibrary(hUser32);
	}
	catch (...)
	{
		pData->dwError = pData->DynGetLastError();
		return ERROR_UNKNOWN;
	}

	return 0;
}

#endif
